' --------------------------------------------------------------------------------------------------
' thies.bas ist ein Basic-Script das in Verbindung mit dem Serial Scriptable Port (elviscpd)' des Visualisierungssystems Elvis eingesetzt werden kann.
' Es implementiert die Steuerrung einer Thies Wetterstation.
'
' Mehr zu Thies Clima unter http://www.thiesclima.com/
' 	
' Die Parameter fr die Schnittstelle knnen ber die Projektierung angegeben werden. Das Trennzeichen ist der "/".
' Beispiel: OpenParam=port=COM5/baud=19200/parity=n/data=8/Stop=1/xon=off/rts=off
' 
' Historie:

' 15.05.2006, Holger Schnauer: Erstellung des Scriptes
' 16.05.2006, Holger Schnauer: Erweiterung (Prozeduren: FillWeatherData & SendWeatherData)
' 16.05.2006, Harald Pokorny: Erweiterung (Funktion:	 FindHeaderAndCleanAccu,OnReceive,RequestValue)
' Funktionsweise des Scriptes: Das Script erhlt auf Anfrage des Benutzers einen Datenstring von der Thies Wetterstation.
' Dieser String wird in einer Array gespeichert.Die Zerlegung und Zuordnung der Messdaten geschieht in der 
' Prozedur "FillWeatherData".Hier werden die Daten geordnet in der "WetterDaten" Struktur gespeichert.
' Im Anschluss gelangen sie durch die Prozedur "SendWeatherData" zu den zugehrigen Adressen in ELVIS.   
' Copyright (C) 2006, IT GmbH
' --------------------------------------------------------------------------------------------------
' Dieses Makro ist Teil des Softwareprodukts Elvis. Es kann frei verwendet,
' modifiziert und verteilt werden, vorausgesetzt, obige Copyright-Notiz wird nicht entfernt.
' Ein Gewhrleistung fr die korrekte Funktion des Makros wird nicht bernommen.
' --------------------------------------------------------------------------------------------------

Option Explicit
Option Base 0

Global g_A(1023) As Integer 				'Accumulator as Byte Array, length =  max 4 x 255
Global g_AWI As Integer					'write index
Global g_Header As Boolean				'Kennzeichen fr Kopfdaten 

Global Const g_TelegramLen = 78			' die normale Lnge eines Telegramms mit Endekennung
Global Const g_HeaderLen = 206			' die normale Lnge eines Headers mit Endekennung
			'Wert fr die Schwenkgeschwindigkeit
Sub Main()
   	'OnOpen
   	'OnClose
End Sub
Type WetterDaten					'Struktur fr die Wetterdaten
									
	Counter As String				'Anz Messungen (nach 9999 springt der Counter auf 0100 zurck)
	Sichtweite As String 			'Sichtweite in m
	StatusSonne As String		'Sonnenstatus (0 = aus / 1 = an)
	relFeuchte As String			'relative Feuchte
	Strahlung As String 		'Strahlung Sonne
	momWert1 As String			'momentan Wert 1
	momWert2 As String			'momentan Wert 2
	LsWSs As String				'Lfterstrom Wetter Strahlungsschutz
	momTemp1 As String			'momentane Temperatur1 in C
	momTemp2 As String 			'momentane Temperatur2 in C
	momTemp3 As String			'momentane Temperatur3 in C
	momWindRichtung As String	'momentane Windrichtung in 
	momWindGeschw As String		'momentane Windgeschwindigkeit in m/s
	NSStatus As String			'Niederschlagstatus (1 = Niederschlag / 0 = kein Niederschlag)
	SumNS As String				'Summe Niederschlag seit letzter Messung
	Wolkenhoehe As String		'Wolkenhoehe in m

End Type
Public Wetter As WetterDaten 

Sub OnOpen(sParam As String)
	Dim sPortSettings() As String			' parameter of open = port settings (array)
	Dim sPortComSettings As String

    Print "Start OnOpen ------------------------------ version 0.1.1"
    FillPortSettings sPortSettings, sParam, "/"
    Print "Parameter: " & sParam
	'2. parameter of OpenCommPort is mode:
	'baud=<Baudrate>
	'parity=n|e|o|m|s
	'data=5|6|7|8
	'stop=1|1,5|2
	'to=on|off
	'xon=on|off
	'odsr=on|off
	'octs=on|off
	'dtr=on|off|hs
	'rts=on|off|hs|tg
	'idsr=on|off
    
	'OpenCommPort "COM5", "Baud=19200, parity=n, data=8, stop=1, xon=off, rts=off"
	
	sPortComSettings = GetTokenValue(sPortSettings, "baud=", "baud=", "9600")
	sPortComSettings = sPortComSettings & "," & GetTokenValue(sPortSettings, "parity=", "parity=" ,"n")
	sPortComSettings = sPortComSettings & "," & GetTokenValue(sPortSettings, "data=", "data=", "8")
	sPortComSettings = sPortComSettings & "," & GetTokenValue(sPortSettings, "stop=", "stop=", "1")
	sPortComSettings = sPortComSettings & "," & GetTokenValue(sPortSettings, "xon=", "xon=", "off")
	sPortComSettings = sPortComSettings & "," & GetTokenValue(sPortSettings, "rts=", "rts=", "off")	
	
	Print "sPortComSettings: " & sPortComSettings
	
	OpenCommPort GetTokenValue(sPortSettings, "port=", "", "COM1"), sPortComSettings
	g_AWI = 0
	g_Header = False
	
	'Kommunikation der Wetterstation starten
	
	
	
End Sub

Sub OnClose()
    CloseCommPort	'will be closed automatic if this is not done here
End Sub

Function GetTokenValue(sPortSettings() As String, ByVal sToken As String, sPreText As String, sDefault As String) As String
        Dim sPortSetting As String
        Dim nLenToken As Integer
        
        GetTokenValue = sPreText & sDefault
        
        nLenToken = Len(sToken ) 
        'Print "GetTokenValue sPortSettings(1) = " & sPortSettings(1)
        'Print "GetTokenValue sPortSettings(2) = " & sPortSettings(2)
        'Print "GetTokenValue sPortSettings(3) = " & sPortSettings(3)
        'Print "GetTokenValue sPortSettings(4) = " & sPortSettings(4)
        Dim i As Integer
        For i = UBound(sPortSettings) To 1 Step -1
                sPortSetting = sPortSettings(i)
                'Print "sToken = " & sToken
                'Print "Left(sPortSetting, nLenToken) = " & Left(sPortSetting, nLenToken)
                If Left(sPortSetting, nLenToken) = LCase(sToken) Then
                        'Print "GetTokenValue = " & Right(sPortSetting, Len(sPortSetting)-nLenToken)
                        GetTokenValue = sPreText & Right(sPortSetting, Len(sPortSetting) - nLenToken)
                        Exit For
                End If
        Next i
End Function


Sub FillPortSettings(sPortSettings() As String, sInput As String, sep As String)

	Dim nPortSettings As Integer			' Argumente - Zhler
	Dim sPortSetting As String
	Dim iPos As Integer
	Dim sRest As String
	
	If Len(sInput) < 1 Then GoTo ExitFunction
	nPortSettings = 1
	ReDim Preserve sPortSettings(nPortSettings)

	sRest = sInput
	'Print "sRest = " & sRest
                
	iPos = InStr(sRest, sep)
	While iPos > 0
		sPortSetting = Left(sRest, iPos - 1)
		'Print "sPortSetting = " & sPortSetting
		If sPortSetting <> "" Then
			ReDim Preserve sPortSettings(nPortSettings)
			sPortSettings(nPortSettings) = sPortSetting
			nPortSettings = nPortSettings + 1
			'Print "sPortSettings(" & CStr(nPortSettings) & ") = " & sPortSetting
		End If
		sRest = Right(sRest, Len(sRest) - iPos)
		iPos = InStr(sRest, sep)
	Wend
	ReDim Preserve sPortSettings(nPortSettings)
	sPortSettings(nPortSettings) = sRest


ExitFunction:

End Sub

Sub RequestValue(dst As String, svrinfo As String)
		ShiftAccuLeft(g_AWI)			'das Array wird zuruckgesetzt
		
		
		Print "RequestValue for: " & dst & " with svrinfo " & svrinfo 
		
		CommSend &H02      'Strg + B
		CommSend &H6D		 'm		
		CommSend &H6D		 'm
		CommSend &H03		 'Strg + C
End Sub

Sub FillWeatherData() 		'fllt die WetterDaten Struktur mit den Messwerten

With Wetter
	.Counter = Str(Chr(g_A(0)) & Chr(g_A(1)) & Chr(g_A(2)) & Chr(g_A(3)))
	.Sichtweite = Str(Chr(g_A(5)) & Chr(g_A(6)) & Chr(g_A(7)) & Chr(g_A(8)) & Chr(g_A(9)))
	.StatusSonne = Str(Chr(g_A(11)))
	.relFeuchte = Str(Chr(g_A(13)) & Chr(g_A(14)) & Chr(g_A(15)) & Chr(g_A(16)) & Chr(g_A(17)))
	.Strahlung = Str(Chr(g_A(19)) & Chr(g_A(20)) & Chr(g_A(21)) & Chr(g_A(22)))
	.momWert1 = Str(Chr(g_A(24)) & Chr(g_A(25)) & Chr(g_A(26)) & Chr(g_A(27)) & Chr(g_A(28)))
	.momWert2 = Str(Chr(g_A(30)) & Chr(g_A(31)) & Chr(g_A(32)) & Chr(g_A(33)) & Chr(g_A(34)))
	.LsWSs = Str(Chr(g_A(36)) & Chr(g_A(37)) & Chr(g_A(38)) & Chr(g_A(39)))
	.momTemp1 =  Str(Chr(g_A(41)) & Chr(g_A(42)) & Chr(g_A(43)) & Chr(g_A(44)) & Chr(g_A(45)))
	.momTemp2 =  Str(Chr(g_A(47)) & Chr(g_A(48)) & Chr(g_A(49)) & Chr(g_A(50)) & Chr(g_A(51)))
	.momTemp3 =  Str(Chr(g_A(53)) & Chr(g_A(54)) & Chr(g_A(55)) & Chr(g_A(56)) & Chr(g_A(57)))
	.momWindRichtung = Str(Chr(g_A(59)) & Chr(g_A(60)) & Chr(g_A(61)))
	.momWindGeschw =  Str(Chr(g_A(63)) & Chr(g_A(64)) & Chr(g_A(65)) & Chr(g_A(66)))
	.NSStatus =  Str(Chr(g_A(68)))
	.SumNS = Str(Chr(g_A(70)) & Chr(g_A(71)) & Chr(g_A(72)) & Chr(g_A(73)) & Chr(g_A(74)))
End With

End Sub

Sub SendWeatherData()		'sendet die Daten der WetterDaten Struktur an die Adressen in ELVIS		
	If IsNumeric(Wetter.Counter) Then
		FireReceiveValue "counter", CInt(Wetter.Counter)
	Else
		FireReceiveValue "counter", Wetter.Counter				'Hinweis: die Adressen sind frei whlbar!
	End If
	
	If IsNumeric(Wetter.Sichtweite) Then
		FireReceiveValue "sichtweite", CInt(Wetter.Sichtweite)			'Es mssen nicht die Vorhandenen verwendet werden.
	Else
		FireReceiveValue "sichtweite", Wetter.Sichtweite
	End If
	
	If IsNumeric(Wetter.StatusSonne) Then
		FireReceiveValue "statussonne", CInt(Wetter.StatusSonne)
	Else
		FireReceiveValue "statussonne", Wetter.StatusSonne
	End If
	
	If IsNumeric(Wetter.relFeuchte) Then
		FireReceiveValue "relfeuchte", CDbl(Wetter.relFeuchte)
	Else
		FireReceiveValue "relfeuchte", Wetter.relFeuchte
	End If
	
	If IsNumeric(Wetter.Strahlung) Then
		FireReceiveValue "strahlung", CInt(Wetter.Strahlung)
	Else
		FireReceiveValue "strahlung", Wetter.Strahlung
	End If
	
	If IsNumeric(Wetter.momWert1) Then
		FireReceiveValue "momwert1", CDbl(Wetter.momWert1)
	Else
		FireReceiveValue "momwert1", Wetter.momWert1
	End If	
	
	If IsNumeric(Wetter.momWert2) Then
		FireReceiveValue "momwert2", CDbl(Wetter.momWert2)
	Else
		FireReceiveValue "momwert2", Wetter.momWert2
	End If

	If IsNumeric(Wetter.LsWSs) Then
		FireReceiveValue "lswss", CInt(Wetter.LsWSs)
	Else 
		FireReceiveValue "lswss", Wetter.LsWSs
	End If
	
	If IsNumeric(Wetter.momTemp1) Then
		FireReceiveValue "momtemp1", CDbl(Wetter.momTemp1)
	Else
		FireReceiveValue "momtemp1", Wetter.momTemp1
	End If

	If IsNumeric(Wetter.momTemp2) Then
		FireReceiveValue "momtemp2", CDbl(Wetter.momTemp2)
	Else
		FireReceiveValue "momtemp2", Wetter.momTemp2
	End If

	If IsNumeric(Wetter.momTemp3) Then
		FireReceiveValue "momtemp3", CDbl(Wetter.momTemp3)
	Else
		FireReceiveValue "momtemp3", Wetter.momTemp3
	End If

	If IsNumeric(Wetter.momWindRichtung) Then
		FireReceiveValue "momwindrichtung", CInt(Wetter.momWindRichtung)
	Else
		FireReceiveValue "momwindrichtung", Wetter.momWindRichtung
	End If 
	
	If IsNumeric(Wetter.momWindGeschw) Then
		FireReceiveValue "momwindgeschw", CDbl(Wetter.momWindGeschw)
	Else 
		FireReceiveValue "momwindgeschw", Wetter.momWindGeschw
	End If	

	If IsNumeric(Wetter.NSStatus) Then
		FireReceiveValue "nsstatus", CInt(Wetter.NSStatus)
	Else
		FireReceiveValue "nsstatus", Wetter.NSStatus
	End If

	If IsNumeric(Wetter.SumNS) Then
		FireReceiveValue "sumns", CDbl(Wetter.SumNS)
	Else
		FireReceiveValue "sumns", Wetter.SumNS
	End If
End Sub

Sub SendHeaderData()
	Dim i As Integer
	Dim maxi As Integer
	Dim s As String
	
	If g_AWI < g_HeaderLen - 1 Then
		Print "Warnung - SendHeaderData: Kopfdaten sind nicht vollstndig"
		maxi = g_AWI
	Else
		maxi = g_HeaderLen - 1
	End If
	 
	For i = 0 To maxi
		s = s & Str(Chr(g_A(i))) 
	Next i
	FireReceiveValue "kopf", s
End Sub

Function FindHeaderAndCleanAccu() As Boolean		'untersucht,ob es sich bei den gefundenen Daten um einen Header handelt
	Dim i As Integer									'Hinweis: Header = Startinformationen,die beim Einschalten des Gertes
	Dim bHeader As Boolean							'angezeigt werden (CPU Takt,Versionsnummer ...)
	
	bHeader = False
	Do While i < g_AWI
		If g_A(i) = &H0D And g_A(i+1) = &H0A And g_A(i+2) = &H2D And g_A(i+3) = &H2D And _
			g_A(i+4) = &H2D And g_A(i+5) = &H2D And g_A(i+6) = &H2D And g_A(i+7) = &H2D Then
			bHeader = True
			Exit Do			
		End If
		i = i + 1		
	Loop	
	If bHeader And i > 0 Then ShiftAccuLeft(i-1)
	FindHeaderAndCleanAccu = bHeader 
End Function

Sub OnReceive(ByteData As Byte)
	Dim dst As String
	Dim  v As String 
	Dim bCommand As Boolean

	g_A(g_AWI) = ByteData
	g_AWI = g_AWI + 1

	g_Header = FindHeaderAndCleanAccu()
	
	Print "Receive: <" & CStr(ByteData) & ">"
	
	bCommand = True 
	While bCommand		
		If g_Header Then								'wenn es sich um einen Header handelt,wird dieser gelscht,oder er 
			If g_AWI >= g_HeaderLen Then			'kann zur Anzeige gebracht werden	(optional)
				'header anzeigen
				SendHeaderData
				ShiftAccuLeft(g_HeaderLen)			'die Array wird zuruckgesetzt
				'bCommand = False
			Else
				bCommand = False
			End If
		Else
			If g_AWI >= g_TelegramLen Then			'erst wenn das Array seine entgltige Gre erreicht hat
				FillWeatherData						'werden die Prozeduren zur Weiterverarbeitung aufgerufen
				SendWeatherData
				ShiftAccuLeft(g_TelegramLen)		'die Array wird zuruckgesetzt
			Else
				bCommand = False
			End If
		End If
	Wend
End Sub


Sub ShiftAccuLeft(ByVal nPos As Integer)		'a helpful little function if the accu is used - some bigger example show it a better way - call for them!
	Dim i As Integer
	For i = 0 To g_AWI - 1
		g_A(i) = g_A(i + nPos)
	Next i
	g_AWI = g_AWI - nPos
End Sub












